#! /usr/bin/env bash
# ******************************************************
# Author        :   www.aiops.com
# Last modified :   2020-10-19 09:47
# Filename      :   install.sh
# Version       :   3.0.1
# Description   :   Install alert script for zabbix.
# ******************************************************

set -e

CUR_DIR=$(cd `dirname $0`; pwd)
C_GRE='\e[0;32m'
C_RED='\e[0;31m'
END='\e[0m'

#该探针版本号1200 第一个1代表探针的类型（zabbix新版本探针） 200代表该类型的版本
AGENT_VERSION=1200

APPKEY="$1"
if [ -z $APPKEY ];then
	echo -e "${C_RED}缺少AppKey，请确认后重新执行安装！${END}"
	exit 1
fi

if [ ! -s ${CUR_DIR}/funcs ];then
	echo -e "${C_RED}缺少函数文件，请检查探针组件！${END}"
	exit 1
fi
. ${CUR_DIR}/funcs

$(log INFO Install "Start to install zabbix agent of CloudAlert")
$(log INFO Install "Your AppKey is \"$APPKEY\"")

UNINSTALL_SCRIPT=${CUR_DIR}/uninstall.sh
SCRIPT_RELATIVE_PATH="cloudalert/bin/alert.sh"
MEDIA_NAME="Cloud Alert Media"
GROUP_NAME="Cloud Alert Group"
USER_NAME="Cloud Alert User"
USER_PASSWD="password"
ACTION_NAME="Cloud Alert Action"
TRIGGER_SUBJECT="trigger subject"
RESOLVE_SUBJECT="resolve subject"
VER_MAJOR=4
ZABBIX_URL=""
AUTH_ID=""
MEDIA_ID=""
GROUP_ID=""
USER_ID=""
ACTION_ID=""

# check connection
$(log INFO Install "Start to check connection")
while true
do
	read -p "请输入Zabbix管理地址：" ZABBIX_HOST
	CHECK_CODE=$(curl -I -m 10 -o /dev/null -s -w %{http_code} $ZABBIX_HOST)
	if [ 200 -eq $CHECK_CODE -o 301 -eq $CHECK_CODE ];then
		ZABBIX_URL="$ZABBIX_HOST/api_jsonrpc.php"
		break
	fi
	
	echo -e "${C_RED}连接Zabbix异常，请重新输入Zabbix管理地址！${END}"
done
echo -e "${C_GRE}Check connection success!${END}"
CMD="sed -i '/^ZABBIX_URL=/c ZABBIX_URL=$ZABBIX_URL' $UNINSTALL_SCRIPT"
eval $CMD

# authentication
$(log INFO Install "Start to auth by username and password")
while true
do
	read -p "请输入Zabbix管理员账号：" ZABBIX_USER
	read -s -p "请输入Zabbix管理员密码：" ZABBIX_PASSWD
	AUTH_REQ="{\"jsonrpc\": \"2.0\",\"method\": \"user.login\",\"id\": \"1\",\"params\": {\"user\": \"$ZABBIX_USER\",\"password\": \"$ZABBIX_PASSWD\"}}"
	AUTH_RESP=$(http_post "$AUTH_REQ" "$ZABBIX_URL")
	$(log INFO Install "Auth response: $AUTH_RESP")
	if [ -z "$AUTH_RESP" ];then
		echo -e "${C_RED}认证时出现异常：$AUTH_REQ${END}"
		exit 1
	fi
	AUTH_ID=$(parse_json "$AUTH_RESP" "result")
	if [ "error" != "$AUTH_ID" ];then
		CMD="sed -i '/^USERNAME=/c USERNAME=$ZABBIX_USER' $UNINSTALL_SCRIPT"
		eval $CMD
		CMD="sed -i '/^PASSWORD=/c PASSWORD=$ZABBIX_PASSWD' $UNINSTALL_SCRIPT"
		eval $CMD
		$(log INFO Install "Auth success by user: $ZABBIX_USER, password: $ZABBIX_PASSWD")
		break
	fi
	
	echo -e "${C_RED}认证失败，请重新输入管理员账号及密码！${END}"
done
echo -e "${C_GRE}Authentication success!${END}"

# check zabbix api version
$(log INFO Install "Start to check zabbix api version")
VERSION_REQ="{\"jsonrpc\": \"2.0\",\"method\": \"apiinfo.version\",\"params\": [],\"id\": 1}"
VERSION_RESP=$(http_post "$VERSION_REQ" "$ZABBIX_URL")
$(log INFO Install "Check zabbix api version response: $VERSION_RESP")
if [ -n "$VERSION_RESP" ];then
	VERSION_CODE=$(parse_json "$VERSION_RESP" "result")
	if [ "error" != "$VERSION_CODE" ];then
		VER_MAJOR=$(echo $VERSION_CODE | awk -F '.' '{print $1}')
		$(log INFO Install "Zabbix api version: $VERSION_CODE")
	fi
fi
echo -e "${C_GRE}Check api version success!${END}"

# create zabbix media type
$(log INFO Install "Start to create new media type")
MEDIA_REQ="{\"jsonrpc\": \"2.0\",\"method\": \"mediatype.create\",\"params\": {\"name\": \"$MEDIA_NAME\",\"description\": \"$MEDIA_NAME\",\"type\": 1,\"exec_params\": \"{ALERT.MESSAGE}\n\",\"exec_path\": \"$SCRIPT_RELATIVE_PATH\"},\"auth\": \"$AUTH_ID\",\"id\": 1}"
MEDIA_RESP=$(http_post "$MEDIA_REQ" "$ZABBIX_URL")
$(log INFO Install "Create media type response: $MEDIA_RESP")
if [ -z "$MEDIA_RESP" ];then
	echo -e "${C_RED}创建新报警媒介类型时出现异常：$MEDIA_REQ${END}"
	exit 1
fi
MEDIA_CODE=$(parse_json "$MEDIA_RESP" "mediatypeids")
if [ "error" == "$MEDIA_CODE" ];then
	echo -e "${C_RED}创建新报警媒介类型失败！错误消息：$MEDIA_RESP！${END}"
	exit 1
fi
MEDIA_ID=$(parse_id $MEDIA_CODE)
echo -e "${C_GRE}Create new media type success!${END}"
$(log INFO Install "Create new media type success!")
CMD="sed -i '/^MEDIA_ID=/c MEDIA_ID=$MEDIA_ID' $UNINSTALL_SCRIPT"
eval $CMD

# create zabbix user group
$(log INFO Install "Start to create new user group")
GROUP_REQ="{\"jsonrpc\": \"2.0\",\"method\": \"usergroup.create\",\"params\": {\"name\": \"$GROUP_NAME\"},\"auth\": \"$AUTH_ID\",\"id\": 1}"
GROUP_RESP=$(http_post "$GROUP_REQ" "$ZABBIX_URL")
$(log INFO Install "Create user group response: $GROUP_RESP")
if [ -z "$GROUP_RESP" ];then
	echo -e "${C_RED}创建新用户群组时出现异常：$MSG_BODY${END}"
	exit 1
fi
GROUP_CODE=$(parse_json "$GROUP_RESP" "usrgrpids")
if [ "error" == "$GROUP_CODE" ];then
	echo -e "${C_RED}创建新用户群组失败！错误消息：$GROUP_RESP${END}"
	exit 1
fi
GROUP_ID=$(parse_id $GROUP_CODE)
echo -e "${C_GRE}Create new group success!${END}"
$(log INFO Install "Create new group success!")
CMD="sed -i '/^GROUP_ID=/c GROUP_ID=$GROUP_ID' $UNINSTALL_SCRIPT"
eval $CMD

# create new user for zabbix
$(log INFO Install "Start to create new user")
USER_REQ="{\"jsonrpc\": \"2.0\",\"method\": \"user.create\",\"params\": {\"alias\": \"$USER_NAME\",\"name\": \"$USER_NAME\",\"passwd\": \"$USER_PASSWD\",\"type\": 3,\"usrgrps\": [{\"usrgrpid\": \"$GROUP_ID\"}],\"user_medias\": [{\"mediatypeid\": \"$MEDIA_ID\",\"sendto\": \"$USER_NAME\",\"active\": 0,\"severity\": 63,\"period\": \"1-7,00:00-24:00\"}]}, \"auth\": \"$AUTH_ID\",\"id\": 1}"
USER_RESP=$(http_post "$USER_REQ" "$ZABBIX_URL")
$(log INFO Install "Create user response: $USER_RESP")
if [ -z "$USER_RESP" ];then
	echo -e "${C_RED}创建新用户时出现异常：$USER_REQ${END}"
	exit 1
fi
USER_CODE=$(parse_json "$USER_RESP" "userids")
if [ "error" == "$USER_CODE" ];then
	echo -e "${C_RED}创建新用户失败！错误消息：$USER_RESP${END}"
	exit 1
fi
USER_ID=$(parse_id $USER_CODE)
echo -e "${C_GRE}Create new user success!${END}"
$(log INFO Install "Create new user success!")
CMD="sed -i '/^USER_ID=/c USER_ID=$USER_ID' $UNINSTALL_SCRIPT"
eval $CMD

# create new action
$(log INFO Install "Start to create new action")
ACTION_REQ="{}"
CUSTOM_TRIGGER="{'app': '$APPKEY','eventId': '{HOST.HOST}-{TRIGGER.ID}','eventType': 'trigger','alarmName': '{TRIGGER.NAME}','entityName': '{HOSTNAME}','entityId': '{HOST.HOST}-{TRIGGER.ID}','alarmContent': '{HOST.HOST}{ITEM.NAME}:{ITEM.VALUE}{TRIGGER.NAME}','priority': '{TRIGGER.NSEVERITY}','host': '{HOST.HOST}','tag': [{'hostgroups': ['{TRIGGER.HOSTGROUP.NAME}']}],'agentVersion': '$AGENT_VERSION','service': '{ITEM.NAME}','ip': '{HOST.IP}','itemName': '{ITEM.NAME}','itemValue': '{ITEM.VALUE}'}"
CUSTON_RESOLVE="{'app': '$APPKEY','eventId': '{HOST.HOST}-{TRIGGER.ID}','eventType': 'resolve','alarmName': '{TRIGGER.NAME}','entityName': '{HOSTNAME}','entityId': '{HOST.HOST}-{TRIGGER.ID}','alarmContent': '{HOST.HOST}{ITEM.NAME}:{ITEM.VALUE}{TRIGGER.NAME}','priority': '{TRIGGER.NSEVERITY}','host': '{HOST.HOST}','tag': [{'hostgroups': ['{TRIGGER.HOSTGROUP.NAME}']}],'agentVersion': '$AGENT_VERSION','service': '{ITEM.NAME}','ip': '{HOST.IP}','itemName': '{ITEM.NAME}','itemValue': '{ITEM.VALUE}'}"

if [ 5 -eq $VER_MAJOR ];then
	ACTION_REQ="{\"jsonrpc\": \"2.0\",\"method\": \"action.create\",\"params\": {\"name\": \"$ACTION_NAME\",\"eventsource\": 0,\"status\": 0,\"esc_period\": \"3600\",\"operations\": [{\"operationtype\": 0,\"opmessage_grp\": [{\"usrgrpid\": \"$GROUP_ID\"}],\"opmessage_usr\": [{\"userid\": \"$USER_ID\"}],\"opmessage\": {\"default_msg\": 0,\"subject\": \"$TRIGGER_SUBJECT\",\"message\": \"$CUSTOM_TRIGGER\",\"mediatypeid\": \"$MEDIA_ID\"}}],\"recovery_operations\":[{\"operationtype\": 0,\"opmessage_grp\": [{\"usrgrpid\": \"$GROUP_ID\"}],\"opmessage_usr\": [{\"userid\": \"$USER_ID\"}],\"opmessage\": {\"default_msg\": 0,\"subject\": \"$RESOLVE_SUBJECT\",\"message\": \"$CUSTON_RESOLVE\",\"mediatypeid\": \"$MEDIA_ID\"}}]},\"auth\": \"$AUTH_ID\", \"id\": 1}"
else
	ACTION_REQ="{\"jsonrpc\": \"2.0\",\"method\": \"action.create\",\"params\": {\"name\": \"$ACTION_NAME\",\"eventsource\": 0,\"status\": 0,\"recovery_msg\": 1,\"esc_period\": \"3600\",\"def_longdata\": \"$CUSTOM_TRIGGER\",\"def_shortdata\": \"$TRIGGER_SUBJECT\",\"r_longdata\": \"$CUSTON_RESOLVE\",\"r_shortdata\": \"$RESOLVE_SUBJECT\",\"operations\":[{\"operationtype\": 0,\"opmessage_usr\": [{\"userid\": \"$USER_ID\"}],\"opmessage\": {\"default_msg\": 1}}],\"recovery_operations\": [{\"operationtype\": 0,\"opmessage_usr\": [{\"userid\": \"$USER_ID\"}],\"opmessage\": {\"default_msg\": 1}}]},\"auth\": \"$AUTH_ID\", \"id\": 1}"
fi
ACTION_RESP=$(http_post "$ACTION_REQ" "$ZABBIX_URL")
$(log INFO Install "Create action response: $ACTION_RESP")
if [ -z "$ACTION_RESP" ];then
	echo -e "${C_RED}创建新动作时出现异常：$ACTION_REQ${END}"
	exit 1
fi
ACTION_CODE=$(parse_json "$ACTION_RESP" "actionids")
if [ "error" == "$ACTION_CODE" ];then
	echo -e "${C_RED}创建新动作失败！错误消息：$ACTION_RESP${END}"
	exit 1
fi
ACTION_ID=$(parse_id $ACTION_CODE)
echo -e "${C_GRE}Create new action success!${END}"
$(log INFO Install "Create new action success!")
CMD="sed -i '/^ACTION_ID=/c ACTION_ID=$ACTION_ID' $UNINSTALL_SCRIPT"
eval $CMD

echo -e "${C_GRE}Install zabbix agent success!${END}"
$(log INFO Install "Install zabbix agent success!")

exit 0
