#!/usr/bin/env bash
# ******************************************************
# Author        :   www.aiops.com
# Last modified :   2020-10-21 09:35
# Filename      :   uninstall.sh
# Version       :   3.0.1
# Description   :   Uninstall zabbix agent script.
# ******************************************************

set -e

CUR_DIR=$(cd `dirname $0`; pwd)

C_GRE='\e[0;32m'
C_RED='\e[0;31m'
END='\e[0m'

if [ ! -s ${CUR_DIR}/funcs ];then
	echo -e "${C_RED}缺少函数文件，请检查探针组件！${END}"
	exit 1
fi
. ${CUR_DIR}/funcs

$(log INFO Uninstall "Start to uninstall zabbix agent of CloudAlert")

ZABBIX_URL=
USERNAME=
PASSWORD=
ACTION_ID=
USER_ID=
GROUP_ID=
MEDIA_ID=

# authentication
$(log INFO Uninstall "Start to auth by username and password")
AUTH_REQ="{\"jsonrpc\": \"2.0\",\"method\": \"user.login\",\"id\": \"1\",\"params\": {\"user\": \"$USERNAME\",\"password\": \"$PASSWORD\"}}"
AUTH_RESP=$(http_post "$AUTH_REQ" "$ZABBIX_URL")
if [ -z "$AUTH_RESP" ];then
	echo -e "${C_RED}认证时出现异常：$AUTH_REQ${END}"
	exit 1
fi
AUTH_ID=$(parse_json "$AUTH_RESP" "result")
if [ "error" == "$AUTH_ID" ];then
	echo -e "${C_RED}认证失败，请重新输入管理员账号及密码！${END}"
	exit 1
fi
echo -e "${C_GRE}Authentication success!${END}"
$(log INFO Uninstall "Authentication success!")

OBJECTS="action user usergroup mediatype"
for item in $OBJECTS
do
	$(log INFO Uninstall "Delete zabbix configuration: ${item}")
	ID=0
	RESULTS=""
	case $item in
		action)
			ID=$ACTION_ID
			RESULTS="actionids"
			;;
		user)
			ID=$USER_ID
			RESULTS="userids"
			;;
		usergroup)
			ID=$GROUP_ID
			RESULTS="usrgrpids"
			;;
		mediatype)
			ID=$MEDIA_ID
			RESULTS="mediatypeids"
			;;
	esac

	ITEM_REQ="{\"jsonrpc\": \"2.0\",\"method\": \"${item}.delete\",\"params\": [\"$ID\"],\"auth\": \"$AUTH_ID\", \"id\": 1}"
	ITEM_RESP=$(http_post "$ITEM_REQ" "$ZABBIX_URL")
	if [ -z "$ITEM_RESP" ];then
		echo -e "${C_RED}删除${item}时出现异常：$ITEM_REQ${END}"
	else
		ITEM_CODE=$(parse_json "$ITEM_RESP" "$RESULTS")
		if [ "error" != "$ITEM_CODE" ];then
			DELETE_ID=$(parse_id $ITEM_CODE)
			if [ $ID -eq $DELETE_ID ];then
				echo -e "${C_GRE}Delete ${item} succcess！${END}"
				$(log INFO Uninstall "Delete ${item} succcess！")
			else
				echo -e "${C_RED}删除${item}出错！"
			fi
		else
			echo -e "${C_RED}删除${item}失败！错误消息：$ITEM_RESP${END}"
		fi
	fi
done

echo -e "${C_GRE}Uninstall zabbix agent success!${END}"
$(log INFO Uninstall "Uninstall zabbix agent success!")

exit 0
